@@ -1,3 +1,4 @@
+import argcomplete
 import argparse
 import os
 import re
@@ -5,6 +6,7 @@ import sys
 import textwrap
 
 from . import __version__
+from .project import Project
 
 class ReferenceArchitectureOption:
     choices = ['EDB-RA-1', 'EDB-RA-2', 'EDB-RA-3']
@@ -37,6 +39,15 @@ class PgVersionOption:
     """)
 
 
+class EFMVersionOption:
+    choices = ['3.10', '4.0', '4.1']
+    default = '4.1'
+    help = textwrap.dedent("""
+        EDB Failover Manager version. Allowed values are: 3.10, 4.0 and 4.1.
+        Default: %(default)s
+    """)
+
+
 class PgTypeOption:
     choices = ['PG', 'EPAS']
     default = 'PG'
@@ -170,6 +181,12 @@ def aws_subcommands(aws_subparser):
     aws_show = aws_subparser.add_parser(
         'show', help='Show configuration'
     )
+    aws_display = aws_subparser.add_parser(
+        'display', help='Display project details'
+    )
+    aws_passwords = aws_subparser.add_parser(
+        'passwords', help='Display project password'
+    )
     aws_list = aws_subparser.add_parser(
         'list', help='List projects'
     )
@@ -186,7 +203,7 @@ def aws_subcommands(aws_subparser):
     aws_configure.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     aws_configure.add_argument(
         '-a', '--reference-architecture',
         dest='reference_architecture',
@@ -202,7 +219,7 @@ def aws_subcommands(aws_subparser):
         type=EDBCredentialsType,
         metavar='"<username>:<password>"',
         help="EDB Packages repository credentials."
-    )
+    ).completer = edb_credentials_completer
     aws_configure.add_argument(
         '-o', '--os',
         dest='operating_system',
@@ -227,6 +244,14 @@ def aws_subcommands(aws_subparser):
         metavar='<postgres-version>',
         help=PgVersionOption.help
     )
+    aws_configure.add_argument(
+        '-e', '--efm-version',
+        dest='efm_version',
+        choices=EFMVersionOption.choices,
+        default=EFMVersionOption.default,
+        metavar='<efm-version>',
+        help=EFMVersionOption.help
+    )
     aws_configure.add_argument(
         '-k', '--ssh-pub-key',
         dest='ssh_pub_key',
@@ -258,7 +283,7 @@ def aws_subcommands(aws_subparser):
         default=AWSIAMIDOption.default,
         metavar='<aws-ami-id>',
         help=AWSIAMIDOption.help
-    )
+    ).completer = aws_ami_id_completer
     aws_configure.add_argument(
         '-s', '--spec',
         dest='spec_file',
@@ -270,7 +295,7 @@ def aws_subcommands(aws_subparser):
     aws_logs.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     aws_logs.add_argument(
         '-t', '--tail',
         dest='tail',
@@ -281,27 +306,37 @@ def aws_subcommands(aws_subparser):
     aws_remove.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # aws show sub-command options
     aws_show.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
+    # aws display sub-command option
+    aws_display.add_argument(
+        'project', type=ProjectType, metavar='<project-name>',
+        help='Project name'
+    ).completer = project_name_completer
+    # aws password sub-command option
+    aws_passwords.add_argument(
+        'project', type=ProjectType, metavar='<project-name>',
+        help='Project name'
+    ).completer = project_name_completer
     # aws provision sub-command options
     aws_provision.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # aws destroy sub-command options
     aws_destroy.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # aws deploy sub-command options
     aws_deploy.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     aws_deploy.add_argument(
         '-n', '--no-install-collection',
         dest='no_install_collection',
@@ -326,6 +361,12 @@ def azure_subcommands(azure_subparser):
     azure_show = azure_subparser.add_parser(
         'show', help='Show configuration'
     )
+    azure_display = azure_subparser.add_parser(
+        'display', help='Display project details'
+    )
+    azure_passwords = azure_subparser.add_parser(
+        'passwords', help='Display project passwords'
+    )
     azure_list = azure_subparser.add_parser(
         'list', help='List projects'
     )
@@ -342,7 +383,7 @@ def azure_subcommands(azure_subparser):
     azure_configure.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Terraform project name'
-    )
+    ).completer = project_name_completer
     azure_configure.add_argument(
         '-a', '--reference-architecture',
         dest='reference_architecture',
@@ -358,7 +399,7 @@ def azure_subcommands(azure_subparser):
         type=EDBCredentialsType,
         metavar='"<username>:<password>"',
         help="EDB Packages repository credentials."
-    )
+    ).completer = edb_credentials_completer
     azure_configure.add_argument(
         '-o', '--os',
         dest='operating_system',
@@ -383,6 +424,14 @@ def azure_subcommands(azure_subparser):
         metavar='<postgres-version>',
         help=PgVersionOption.help
     )
+    azure_configure.add_argument(
+        '-e', '--efm-version',
+        dest='efm_version',
+        choices=EFMVersionOption.choices,
+        default=EFMVersionOption.default,
+        metavar='<efm-version>',
+        help=EFMVersionOption.help
+    )
     azure_configure.add_argument(
         '-k', '--ssh-pub-key',
         dest='ssh_pub_key',
@@ -418,7 +467,7 @@ def azure_subcommands(azure_subparser):
     azure_logs.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     azure_logs.add_argument(
         '-t', '--tail',
         dest='tail',
@@ -429,27 +478,37 @@ def azure_subcommands(azure_subparser):
     azure_remove.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # azure show sub-command options
     azure_show.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
+    # azure display sub-command options
+    azure_display.add_argument(
+        'project', type=ProjectType, metavar='<project-name>',
+        help='Project name'
+    ).completer = project_name_completer
+    # azure password sub-command options
+    azure_passwords.add_argument(
+        'project', type=ProjectType, metavar='<project-name>',
+        help='Project name'
+    ).completer = project_name_completer
     # azure provision sub-command options
     azure_provision.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # azure destroy sub-command options
     azure_destroy.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # azure deploy sub-command options
     azure_deploy.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     azure_deploy.add_argument(
         '-n', '--no-install-collection',
         dest='no_install_collection',
@@ -474,6 +533,12 @@ def gcloud_subcommands(gcloud_subparser):
     gcloud_show = gcloud_subparser.add_parser(
         'show', help='Show configuration'
     )
+    gcloud_display = gcloud_subparser.add_parser(
+        'display', help='Display project details'
+    )
+    gcloud_passwords = gcloud_subparser.add_parser(
+        'passwords', help='Display project passwords'
+    )
     gcloud_list = gcloud_subparser.add_parser(
         'list', help='List projects'
     )
@@ -490,7 +555,7 @@ def gcloud_subcommands(gcloud_subparser):
     gcloud_configure.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Terraform project name'
-    )
+    ).completer = project_name_completer
     gcloud_configure.add_argument(
         '-a', '--reference-architecture',
         dest='reference_architecture',
@@ -506,7 +571,7 @@ def gcloud_subcommands(gcloud_subparser):
         type=EDBCredentialsType,
         metavar='"<username>:<password>"',
         help="EDB Packages repository credentials."
-    )
+    ).completer = edb_credentials_completer
     gcloud_configure.add_argument(
         '-o', '--os',
         dest='operating_system',
@@ -531,6 +596,14 @@ def gcloud_subcommands(gcloud_subparser):
         metavar='<postgres-version>',
         help=PgVersionOption.help
     )
+    gcloud_configure.add_argument(
+        '-e', '--efm-version',
+        dest='efm_version',
+        choices=EFMVersionOption.choices,
+        default=EFMVersionOption.default,
+        metavar='<efm-version>',
+        help=EFMVersionOption.help
+    )
     gcloud_configure.add_argument(
         '-k', '--ssh-pub-key',
         dest='ssh_pub_key',
@@ -577,12 +650,12 @@ def gcloud_subcommands(gcloud_subparser):
         type=str,
         metavar='<gcloud-project-id>',
         help="GCloud project ID"
-    )
+    ).completer = gcloud_project_id_completer
     # gcloud logs sub-command options
     gcloud_logs.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     gcloud_logs.add_argument(
         '-t', '--tail',
         dest='tail',
@@ -593,27 +666,37 @@ def gcloud_subcommands(gcloud_subparser):
     gcloud_remove.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # gcloud show sub-command options
     gcloud_show.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
+    # gcloud display sub-command options
+    gcloud_display.add_argument(
+        'project', type=ProjectType, metavar='<project-name>',
+        help='Project name'
+    ).completer = project_name_completer
+    # gcloud password sub-command options
+    gcloud_passwords.add_argument(
+        'project', type=ProjectType, metavar='<project-name>',
+        help='Project name'
+    ).completer = project_name_completer
     # gcloud provision sub-command options
     gcloud_provision.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # gcloud destroy sub-command options
     gcloud_destroy.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     # gcloud deploy sub-command options
     gcloud_deploy.add_argument(
         'project', type=ProjectType, metavar='<project-name>',
         help='Project name'
-    )
+    ).completer = project_name_completer
     gcloud_deploy.add_argument(
         '-n', '--no-install-collection',
         dest='no_install_collection',
@@ -621,6 +704,27 @@ def gcloud_subcommands(gcloud_subparser):
         help="Do not install the Ansible collection."
     )
 
+# Argcomplete completer
+def project_name_completer(prefix, parsed_args, **kwargs):
+    cloud = parsed_args.cloud
+    sub_command = parsed_args.sub_command
+    if sub_command == 'configure':
+        return ("PROJECT_NAME",)
+    else:
+        projects_path = os.path.join(Project.projects_root_path, cloud)
+        if not os.path.exists(projects_path):
+            return ("PROJECT_NAME",)
+        return (pname for pname in os.listdir(projects_path))
+
+def edb_credentials_completer(prefix, parsed_args, **kwargs):
+    return ("USERNAME:PASSWORD",)
+
+def aws_ami_id_completer(prefix, parsed_args, **kwargs):
+    return ("AWS_AMI_ID",)
+
+def gcloud_project_id_completer(prefix, parsed_args, **kwargs):
+    return ("GCLOUD_PROJECT_ID",)
+
 def parse():
     parser = CLIParser(
         description='EDB deployment script for aws, azure and gcloud'
@@ -659,6 +763,9 @@ def parse():
     )
     gcloud_subcommands(gcloud_subparser)
 
+    # Autocompletion with argcomplete
+    argcomplete.autocomplete(parser)
+
     # Parse the arguments and options
     env = parser.parse_args()
 
