@@ -455,6 +455,7 @@ class Project:
             cluster_name=self.name,
             pg_type=env.postgres_type,
             pg_version=env.postgres_version,
+            efm_version=env.efm_version,
             yum_username=edb_repo_username,
             yum_password=edb_repo_password,
             ssh_user=os_spec['ssh_user'],
@@ -608,6 +609,7 @@ class Project:
         extra_vars=dict(
             pg_type=self.ansible_vars['pg_type'],
             pg_version=self.ansible_vars['pg_version'],
+            efm_version=self.ansible_vars['efm_version'],
             yum_username=self.ansible_vars['yum_username'],
             yum_password=self.ansible_vars['yum_password'],
             pass_dir=os.path.join(self.project_path, '.edbpass')
@@ -638,6 +640,51 @@ class Project:
         # Display inventory informations
         self.display_inventory(inventory_data)
 
+    def display_passwords(self):
+        try:
+            states = self.load_states()
+        except Exception as e:
+            states={}
+        status = states.get('ansible', 'UNKNOWN')
+        if status in ['DEPLOYED', 'DEPLOYING']:
+            if status == 'DEPLOYING':
+                print("WARNING: project is in deploying state")
+            pass_dir=os.path.join(self.project_path, '.edbpass')
+            rows = []
+            for pass_file in os.listdir(pass_dir):
+                if pass_file.endswith("_pass"):
+                    user_name = pass_file.replace('_pass','')
+                    with open(
+                        os.path.join(
+                            pass_dir,
+                            pass_file
+                        )
+                    ) as f:
+                        user_password = f.read()
+                    rows.append([
+                        user_name,
+                        user_password.replace('\n','')
+                    ])
+            Project.display_table(
+                ['Username','Password'],
+                rows
+            )
+
+    def display_details(self):
+        try:
+            states = self.load_states()
+        except Exception as e:
+            states={}
+        status = states.get('ansible', 'UNKNOWN')
+        if status in ['DEPLOYED', 'DEPLOYING']:
+            if status == 'DEPLOYING':
+                print("WARNING: project is in deploying state")
+            inventory_data = None
+            ansible = AnsibleCli(self.project_path)
+            with AM("Extracting data from the inventory file"):
+                inventory_data = ansible.list_inventory(self.ansible_inventory)
+            self.display_inventory(inventory_data)
+
     def display_inventory(self, inventory_data):
         if not self.ansible_vars:
             self._load_ansible_vars()
@@ -684,8 +731,14 @@ class Project:
     @staticmethod
     def list(cloud):
         projects_path = os.path.join(Project.projects_root_path, cloud)
+        headers = ["Name", "Path", "Machines", "Resources", "Components"]
         rows = []
         try:
+            # Case when projects' path does not yet exist
+            if not os.path.exists(projects_path):
+                Project.display_table(headers, [])
+                return
+
             for project_name in os.listdir(projects_path):
                 project_path = os.path.join(projects_path, project_name)
                 if not os.path.isdir(project_path):
@@ -713,16 +766,13 @@ class Project:
                     states.get('ansible', 'UNKNOWN')
                 ])
 
-            Project.display_table(
-                ["Name", "Path", "Machines", "Resources", "Components"],
-                rows
-            )
+            Project.display_table(headers, rows)
 
         except OSError as e:
             msg = "Unable to list projects in %s" % projects_path
             logging.error(msg)
             logging.exception(str(e))
-            raise ProjectErro(msg)
+            raise ProjectError(msg)
 
     @staticmethod
     def display_table(headers, rows):
